//-----------------------------------------------------------------------------
// File: rt_Lighting.h
//
// Desc: Header file rt_Lighting sample app
//-----------------------------------------------------------------------------
#ifndef AFX_RT_LIGHTING_H__885B5489_B58B_447A_8E8D_E7612A93DCC8__INCLUDED_
#define AFX_RT_LIGHTING_H__885B5489_B58B_447A_8E8D_E7612A93DCC8__INCLUDED_

#include <atlbase.h>


//-----------------------------------------------------------------------------
// Defines, and constants
//-----------------------------------------------------------------------------
// TODO: change "DirectX AppWizard Apps" to your name or the company name
#define DXAPP_KEY        TEXT("Software\\DirectX AppWizard Apps\\rt_Lighting")

// Struct to store the current input state
struct UserInput
{
    // TODO: change as needed
    BOOL bRotateUp;
    BOOL bRotateDown;
    BOOL bRotateLeft;
    BOOL bRotateRight;
};


typedef
INT_PTR CALLBACK
DialogProc(HWND dialog, UINT msg, WPARAM wp, LPARAM lp);

extern void
hsv_to_rgb(float h, float s, float v,
           float &r, float &g, float &b);

enum e_projection_type
{
    PT_ORTHOGRAPHIC = 0,
    PT_PERSPECTIVE = 1
};

//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the 
//       generic functionality needed in all Direct3D samples. CMyD3DApplication 
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
class CMyD3DApplication : public CD3DApplication
{
private:
    void set_projection();
    void clone_teapot(DWORD fvf);
    void update_material_parameters(UINT dialog_id, DialogProc *proc);
    void update_light_parameters(UINT dialog_id, DialogProc *proc);
    static DialogProc material_specular_power_proc;
    static DialogProc light_position_proc;
    static DialogProc light_direction_proc;
    static DialogProc light_range_proc;
    static DialogProc light_attenuation_proc;
    static DialogProc light_cone_proc;
    void update_light_menu(HMENU menu);
    void on_command(HWND window, WPARAM wp, LPARAM lp);

    BOOL                    m_bLoadingApp;          // TRUE, if the app is loading
    CD3DFont*               m_pFont;                // Font for drawing text

    UserInput               m_UserInput;            // Struct for storing user input 

    FLOAT                   m_fWorldRotX;           // World rotation state X-axis
    FLOAT                   m_fWorldRotY;           // World rotation state Y-axis
    D3DLIGHT8 m_light;
    D3DMATERIAL8 m_material;
    CComPtr<ID3DXMesh> m_teapot_nocolors;
    CComPtr<ID3DXMesh> m_teapot;
    D3DCOLOR m_ambient;
    D3DCOLOR m_background;
    D3DMATERIALCOLORSOURCE m_ambient_source;
    D3DMATERIALCOLORSOURCE m_diffuse_source;
    D3DMATERIALCOLORSOURCE m_specular_source;
    D3DMATERIALCOLORSOURCE m_emissive_source;
    DWORD m_clone_fvf;
    D3DSHADEMODE m_shade_mode;
    D3DFILLMODE m_fill_mode;
    e_projection_type m_projection;
    bool m_light_enabled;
    bool m_animate_view;
    bool m_show_text;
    bool m_lighting;
    bool m_local_viewer;
    bool m_normalize_normals;
    bool m_specular_enable;
    bool m_dithering;
    bool m_color_vertex;

protected:
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects();
    HRESULT RestoreDeviceObjects();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();
    HRESULT Render();
    HRESULT FrameMove();
    HRESULT FinalCleanup();
    HRESULT ConfirmDevice( D3DCAPS8*, DWORD, D3DFORMAT );

    HRESULT RenderText();

    void    UpdateInput( UserInput* pUserInput );

    VOID    ReadSettings();
    VOID    WriteSettings();

public:
    LRESULT MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
    CMyD3DApplication();
};


#endif // !defined(AFX_RT_LIGHTING_H__885B5489_B58B_447A_8E8D_E7612A93DCC8__INCLUDED_)
